@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,6 +10,7 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests;
 
 use Carbon\Carbon;
@@ -29,10 +31,9 @@ abstract class AbstractTestCaseWithOldNow extends AbstractTestCase
     protected function tearDown(): void
     {
         Carbon::resetMacros();
-        Carbon::setTestNow();
         Carbon::serializeUsing(null);
+
         CarbonImmutable::resetMacros();
-        CarbonImmutable::setTestNow();
         CarbonImmutable::serializeUsing(null);
 
         parent::tearDown();
