@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,10 +10,14 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\Carbon;
 
-use \DateTime;
+use BadMethodCallException;
 use Carbon\Carbon;
+use DateTime;
+use Generator;
+use InvalidArgumentException;
 use stdClass;
 use Tests\AbstractTestCase;
 
@@ -660,10 +665,9 @@ class IsTest extends AbstractTestCase
 
     public function testIsSameAsWithInvalidArgument()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidArgumentException(
             'Expected null, string, DateTime or DateTimeInterface, stdClass given'
-        );
+        ));
 
         $current = Carbon::createFromDate(2012, 1, 2);
         $current->isSameAs('Y-m-d', new stdClass());
@@ -879,13 +883,18 @@ class IsTest extends AbstractTestCase
         $this->assertTrue(Carbon::hasFormat('2000-07-01T00:00:00+00:00', Carbon::ATOM));
         $this->assertTrue(Carbon::hasFormat('Y-01-30\\', '\\Y-m-d\\\\'));
 
+        // @see https://github.com/briannesbitt/Carbon/issues/2180
+        $this->assertTrue(Carbon::hasFormat('2020-09-01 12:00:00Europe/Moscow', 'Y-m-d H:i:se'));
+
         if (version_compare(PHP_VERSION, '7.3.0-dev', '>=')) {
             // Due to https://bugs.php.net/bug.php?id=75577, proper "v" format support can only works from PHP 7.3.0.
             $this->assertTrue(Carbon::hasFormat('2012-12-04 22:59.32130', 'Y-m-d H:s.vi'));
         }
 
         // Format failure
+        $this->assertFalse(Carbon::hasFormat(null, 'd m Y'));
         $this->assertFalse(Carbon::hasFormat('1975-05-01', 'd m Y'));
+        $this->assertFalse(Carbon::hasFormat('1975-01-30\\', '\\Y-m-d\\\\'));
         $this->assertFalse(Carbon::hasFormat('Foo 21st', 'D jS'));
         $this->assertFalse(Carbon::hasFormat('Sun 51st', 'D jS'));
         $this->assertFalse(Carbon::hasFormat('Sun 21xx', 'D jS'));
@@ -895,14 +904,103 @@ class IsTest extends AbstractTestCase
         $this->assertFalse(Carbon::hasFormat('19-05-01', 'Y-m-d'));
         $this->assertFalse(Carbon::hasFormat('30/12/2019', 'm/d/Y'));
         $this->assertFalse(Carbon::hasFormat('12/30/2019', 'd/m/Y'));
+        $this->assertFalse(Carbon::hasFormat('1975-05-01', 'Y-m-d!'));
+        $this->assertFalse(Carbon::hasFormat('1975-05-01', 'Y-m-d|'));
+        $this->assertFalse(Carbon::hasFormat('1975-05-01', 'Y-*-d'));
+    }
+
+    public function testHasFormatWithModifiers()
+    {
+        $this->assertTrue(Carbon::hasFormatWithModifiers('2021-05-03T00:00:00+02:00', 'Y-m-d\TH:i:sp'));
+        $this->assertTrue(Carbon::hasFormatWithModifiers('2021-05-03T00:00:00+02:00', 'Y-m-d\TH:i:sP'));
+        $this->assertTrue(Carbon::hasFormatWithModifiers('2021-05-03T00:00:00Z', 'Y-m-d\TH:i:sp'));
+        $this->assertTrue(Carbon::hasFormatWithModifiers('1975-05-01', 'Y-m-d!'));
+        $this->assertTrue(Carbon::hasFormatWithModifiers('1975-05-01', 'Y-m-d|'));
+        $this->assertTrue(Carbon::hasFormatWithModifiers('1975-05-01', 'Y-*-d'));
+        $this->assertTrue(Carbon::hasFormatWithModifiers('1975-05-01', 'Y-??-d!'));
+        $this->assertTrue(Carbon::hasFormatWithModifiers('1975-05-01', 'Y#m#d'));
+        $this->assertTrue(Carbon::hasFormatWithModifiers('1975/05/31', 'Y#m#d'));
+
+        $this->assertFalse(Carbon::hasFormatWithModifiers('2021-05-03T00:00:00Z', 'Y-m-d\TH:i:sP'));
+        $this->assertFalse(Carbon::hasFormatWithModifiers('1975/31/05', 'Y#m#d'));
+        $this->assertFalse(Carbon::hasFormatWithModifiers('1975-05-01', 'Y-?-d|'));
+        $this->assertFalse(Carbon::hasFormatWithModifiers('1975--01', 'Y-*-d'));
+        $this->assertFalse(Carbon::hasFormatWithModifiers('1975705-01', 'Y#m#d'));
+    }
+
+    public static function dataForFormatLetters(): Generator
+    {
+        yield ['d'];
+        yield ['D'];
+        yield ['j'];
+        yield ['l'];
+        yield ['N'];
+        yield ['S'];
+        yield ['w'];
+        yield ['z'];
+        yield ['W'];
+        yield ['F'];
+        yield ['m'];
+        yield ['M'];
+        yield ['n'];
+        yield ['t'];
+        yield ['L'];
+        yield ['o'];
+        yield ['Y'];
+        yield ['y'];
+        yield ['a'];
+        yield ['A'];
+        yield ['B'];
+        yield ['g'];
+        yield ['G'];
+        yield ['h'];
+        yield ['H'];
+        yield ['i'];
+        yield ['s'];
+        yield ['u'];
+        yield ['v'];
+        yield ['e'];
+        yield ['I'];
+        yield ['O'];
+        yield ['P'];
+        yield ['T'];
+        yield ['Z'];
+        yield ['U'];
+        yield ['c'];
+        yield ['r'];
+    }
+
+    /**
+     * @dataProvider \Tests\Carbon\IsTest::dataForFormatLetters
+     */
+    public function testHasFormatWithSingleLetter($letter)
+    {
+        $output = Carbon::now()->format($letter);
+
+        if ($output === '1000' && $letter === 'v' && version_compare(PHP_VERSION, '7.2.12', '<')) {
+            $output = '000';
+        }
+
+        $this->assertTrue(Carbon::hasFormat($output, $letter), "'$letter' format should match '$output'");
+    }
+
+    public function testCanBeCreatedFromFormat()
+    {
+        $this->assertTrue(Carbon::canBeCreatedFromFormat('1975-05-01', 'Y-m-d'));
+        $this->assertTrue(Carbon::canBeCreatedFromFormat('12/30/2019', 'm/d/Y'));
+        $this->assertFalse(Carbon::canBeCreatedFromFormat('1975-05-01', 'd m Y'));
+        $this->assertFalse(Carbon::canBeCreatedFromFormat('1975-5-1', 'Y-m-d'));
+        $this->assertFalse(Carbon::canBeCreatedFromFormat('19-05-01', 'Y-m-d'));
+        $this->assertFalse(Carbon::canBeCreatedFromFormat('30/12/2019', 'm/d/Y'));
+        $this->assertFalse(Carbon::canBeCreatedFromFormat('12/30/2019', 'd/m/Y'));
+        $this->assertFalse(Carbon::canBeCreatedFromFormat('5', 'N'));
     }
 
     public function testIsSameFoobar()
     {
-        $this->expectException(\BadMethodCallException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new BadMethodCallException(
             'Method isSameFoobar does not exist.'
-        );
+        ));
 
         /** @var mixed $date */
         $date = Carbon::parse('12:00:00');
@@ -911,10 +1009,9 @@ class IsTest extends AbstractTestCase
 
     public function testIsCurrentFoobar()
     {
-        $this->expectException(\BadMethodCallException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new BadMethodCallException(
             'Method isCurrentFoobar does not exist.'
-        );
+        ));
 
         /** @var mixed $date */
         $date = Carbon::parse('12:00:00');
@@ -959,4 +1056,13 @@ class IsTest extends AbstractTestCase
         $this->assertFalse(Carbon::parse('2019-06-02 12:23:45')->is('August 2019'));
         $this->assertFalse(Carbon::parse('2019-06-02 12:23:45')->is('June 2018'));
     }
+
+    public function testHasFormatWithDots()
+    {
+        $this->assertTrue(Carbon::hasFormat('2020.09.09', 'Y.m.d'));
+        $this->assertFalse(Carbon::hasFormat('2020009009', 'Y.m.d'));
+        $this->assertFalse(Carbon::hasFormat('2020-09-09', 'Y.m.d'));
+        $this->assertFalse(Carbon::hasFormat('2020*09*09', 'Y.m.d'));
+        $this->assertFalse(Carbon::hasFormat('2020k09d09', 'Y.m.d'));
+    }
 }
