@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,15 +10,20 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\Carbon;
 
 use Carbon\Carbon;
+use Carbon\Exceptions\UnitException;
 use DateTimeZone;
+use Exception;
+use Generator;
+use InvalidArgumentException;
 use Tests\AbstractTestCase;
 
 class SettersTest extends AbstractTestCase
 {
-    const SET_UNIT_NO_OVERFLOW_SAMPLE = 1000;
+    public const SET_UNIT_NO_OVERFLOW_SAMPLE = 200;
 
     public function testSingularUnit()
     {
@@ -231,11 +237,31 @@ class SettersTest extends AbstractTestCase
 
         $d->setTimestamp(11);
         $this->assertSame(11, $d->timestamp);
+
+        $d->timestamp = 1600887164.88952298;
+        $this->assertSame('2020-09-23 14:52:44.889523', $d->format('Y-m-d H:i:s.u'));
+
+        $d->setTimestamp(1599828571.23561248);
+        $this->assertSame('2020-09-11 08:49:31.235612', $d->format('Y-m-d H:i:s.u'));
+
+        $d->timestamp = '0.88951247 1600887164';
+        $this->assertSame('2020-09-23 14:52:44.889512', $d->format('Y-m-d H:i:s.u'));
+
+        $d->setTimestamp('0.23561248 1599828571');
+        $this->assertSame('2020-09-11 08:49:31.235612', $d->format('Y-m-d H:i:s.u'));
+
+        $d->timestamp = '0.88951247/1600887164/12.56';
+        $this->assertSame('2020-09-23 14:52:57.449512', $d->format('Y-m-d H:i:s.u'));
+
+        $d->setTimestamp('0.00561248/1599828570--1.23');
+        $this->assertSame('2020-09-11 08:49:31.235612', $d->format('Y-m-d H:i:s.u'));
     }
 
     public function testSetTimezoneWithInvalidTimezone()
     {
-        $this->expectException(\InvalidArgumentException::class);
+        $this->expectExceptionObject(new InvalidArgumentException(
+            'Unknown or bad timezone (sdf)'
+        ));
 
         $d = Carbon::now();
         $d->setTimezone('sdf');
@@ -243,10 +269,9 @@ class SettersTest extends AbstractTestCase
 
     public function testTimezoneWithInvalidTimezone()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidArgumentException(
             'Unknown or bad timezone (sdf)'
-        );
+        ));
 
         /** @var mixed $d */
         $d = Carbon::now();
@@ -267,25 +292,24 @@ class SettersTest extends AbstractTestCase
 
         $this->assertSame('America/Vancouver', $new->getTimezone()->getName());
 
-        $new->timezone = 'UTC';
+        @$new->timezone = 'UTC';
 
         $this->assertSame('UTC', $new->getTimezone()->getName());
 
         /** @var mixed $date */
         $date = new Carbon('2020-01-01', 'America/Vancouver');
 
-        /** @var mixed $new */
         $new = clone $date;
 
         $this->assertSame('America/Vancouver', $date->getTimezone()->getName());
 
-        $date->timezone = 'UTC';
+        @$date->timezone = 'UTC';
 
         $this->assertSame('UTC', $date->getTimezone()->getName());
 
         $this->assertSame('America/Vancouver', $new->getTimezone()->getName());
 
-        $new->timezone = 'UTC';
+        @$new->timezone = 'UTC';
 
         $this->assertSame('UTC', $new->getTimezone()->getName());
 
@@ -326,10 +350,9 @@ class SettersTest extends AbstractTestCase
 
     public function testTimezoneWithInvalidTimezoneSetter()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidArgumentException(
             'Unknown or bad timezone (sdf)'
-        );
+        ));
 
         $d = Carbon::now();
         $d->timezone('sdf');
@@ -337,10 +360,9 @@ class SettersTest extends AbstractTestCase
 
     public function testTzWithInvalidTimezone()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidArgumentException(
             'Unknown or bad timezone (sdf)'
-        );
+        ));
 
         /** @var mixed $d */
         $d = Carbon::now();
@@ -349,10 +371,9 @@ class SettersTest extends AbstractTestCase
 
     public function testTzWithInvalidTimezoneSetter()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidArgumentException(
             'Unknown or bad timezone (sdf)'
-        );
+        ));
 
         $d = Carbon::now();
         $d->tz('sdf');
@@ -377,6 +398,9 @@ class SettersTest extends AbstractTestCase
         $this->assertSame(21600, $d2->getTimestamp() - $d->getTimestamp());
         $this->assertSame('America/Toronto', $d2->tzName);
         $this->assertSame('10:53:12.321654', $d2->format('H:i:s.u'));
+
+        $d = Carbon::parse('2018-03-25 00:53:12.321654 America/Toronto')->shiftTimezone('Europe/Oslo');
+        $this->assertSame('2018-03-25 00:53:12.321654 Europe/Oslo', $d->format('Y-m-d H:i:s.u e'));
     }
 
     public function testTimezoneUsingString()
@@ -429,6 +453,9 @@ class SettersTest extends AbstractTestCase
         $this->assertSame(60, $d->offsetMinutes);
         $this->assertSame(1, $d->offsetHours);
         $this->assertSame(60, $d->utcOffset());
+
+        $d->utcOffset(330);
+        $this->assertSame(330, $d->utcOffset());
     }
 
     public function testSetTimezoneUsingDateTimeZone()
@@ -462,7 +489,9 @@ class SettersTest extends AbstractTestCase
 
     public function testInvalidSetter()
     {
-        $this->expectException(\InvalidArgumentException::class);
+        $this->expectExceptionObject(new InvalidArgumentException(
+            "Unknown setter 'doesNotExit'"
+        ));
 
         /** @var mixed $date */
         $date = Carbon::now();
@@ -470,7 +499,7 @@ class SettersTest extends AbstractTestCase
     }
 
     /**
-     * @dataProvider \Tests\Carbon\SettersTest::dataProviderTestSetTimeFromTimeString
+     * @dataProvider \Tests\Carbon\SettersTest::dataForTestSetTimeFromTimeString
      *
      * @param int    $hour
      * @param int    $minute
@@ -484,16 +513,14 @@ class SettersTest extends AbstractTestCase
         $this->assertCarbon($d, 2016, 2, 12, $hour, $minute, $second);
     }
 
-    public function dataProviderTestSetTimeFromTimeString()
+    public static function dataForTestSetTimeFromTimeString(): Generator
     {
-        return [
-            [9, 15, 30, '09:15:30'],
-            [9, 15, 0, '09:15'],
-            [9, 0, 0, '09'],
-            [9, 5, 3, '9:5:3'],
-            [9, 5, 0, '9:5'],
-            [9, 0, 0, '9'],
-        ];
+        yield [9, 15, 30, '09:15:30'];
+        yield [9, 15, 0, '09:15'];
+        yield [9, 0, 0, '09'];
+        yield [9, 5, 3, '9:5:3'];
+        yield [9, 5, 0, '9:5'];
+        yield [9, 0, 0, '9'];
     }
 
     public function testWeekendDaysSetter()
@@ -532,7 +559,7 @@ class SettersTest extends AbstractTestCase
             $second = mt_rand(0, 59);
             $microsecond = mt_rand(0, 999999);
             $units = ['millennium', 'century', 'decade', 'year', 'quarter', 'month', 'day', 'hour', 'minute', 'second', 'week'];
-            $overflowUnit = $units[mt_rand(0, count($units) - 1)];
+            $overflowUnit = $units[mt_rand(0, \count($units) - 1)];
             $units = [
                 'year' => 10,
                 'month' => 12,
@@ -542,7 +569,7 @@ class SettersTest extends AbstractTestCase
                 'second' => 60,
                 'microsecond' => 1000000,
             ];
-            $valueUnit = array_keys($units)[mt_rand(0, count($units) - 1)];
+            $valueUnit = array_keys($units)[mt_rand(0, \count($units) - 1)];
             $value = mt_rand() > 0.5 ?
                 mt_rand(-9999, 9999) :
                 mt_rand(-60, 60);
@@ -564,8 +591,8 @@ class SettersTest extends AbstractTestCase
             if ($modulo < 0) {
                 $modulo += $units[$valueUnit];
             }
-            if ($date->$valueUnit === $value ||
-                $date->$valueUnit === $modulo ||
+            if ($value === $date->$valueUnit ||
+                $modulo === $date->$valueUnit ||
                 (method_exists($date, "diffInReal$unit") && $$valueUnit - $date->{"diffInReal$unit"}($original, false) === $value) ||
                 $$valueUnit - $date->{"diffIn$unit"}($original, false) === $value
             ) {
@@ -586,12 +613,19 @@ class SettersTest extends AbstractTestCase
                 continue;
             }
 
-            throw new \Exception('Unhandled result for: '.
-                'Carbon::parse('.var_export($original->format('Y-m-d H:i:s.u'), true).', '.
-                var_export($original->timezoneName, true).
-                ')->setUnitNoOverflow('.implode(', ', array_map(function ($value) {
-                    return var_export($value, true);
-                }, [$valueUnit, $value, $overflowUnit])).');'."\nGetting: ".$date->format('Y-m-d H:i:s.u e'));
+            $this->failOperation(
+                $original,
+                $date,
+                $start,
+                $end,
+                'setUnitNoOverflow',
+                $valueUnit,
+                $value,
+                $overflowUnit,
+                $unit,
+                $modulo,
+                $$valueUnit
+            );
         }
 
         $minimum = static::SET_UNIT_NO_OVERFLOW_SAMPLE / 100;
@@ -604,24 +638,35 @@ class SettersTest extends AbstractTestCase
 
     public function testSetUnitNoOverflowInputUnitException()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidArgumentException(
             'Unknown unit \'anyUnit\''
-        );
+        ));
 
         Carbon::now()->setUnitNoOverflow('anyUnit', 1, 'year');
     }
 
     public function testSetUnitNoOverflowOverflowUnitException()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidArgumentException(
             'Unknown unit \'anyUnit\''
-        );
+        ));
 
         Carbon::now()->setUnitNoOverflow('minute', 1, 'anyUnit');
     }
 
+    public function testAddUnitError()
+    {
+        $this->expectExceptionObject(new UnitException(implode("\n", [
+            'Unable to add unit array (',
+            "  0 => 'foobar',",
+            '  1 => 1,',
+            ')',
+        ])));
+
+        $date = Carbon::parse('2021-09-13');
+        @$date->addUnit('foobar', 1);
+    }
+
     public function testAddUnitNoOverflow()
     {
         $results = [
@@ -640,7 +685,7 @@ class SettersTest extends AbstractTestCase
             $second = mt_rand(0, 59);
             $microsecond = mt_rand(0, 999999);
             $units = ['millennium', 'century', 'decade', 'year', 'quarter', 'month', 'day', 'hour', 'minute', 'second', 'week'];
-            $overflowUnit = $units[mt_rand(0, count($units) - 1)];
+            $overflowUnit = $units[mt_rand(0, \count($units) - 1)];
             $units = [
                 'year' => 10,
                 'month' => 12,
@@ -650,7 +695,7 @@ class SettersTest extends AbstractTestCase
                 'second' => 60,
                 'microsecond' => 1000000,
             ];
-            $valueUnit = array_keys($units)[mt_rand(0, count($units) - 1)];
+            $valueUnit = array_keys($units)[mt_rand(0, \count($units) - 1)];
             $value = mt_rand() > 0.5 ?
                 mt_rand(-9999, 9999) :
                 mt_rand(-60, 60);
@@ -672,8 +717,8 @@ class SettersTest extends AbstractTestCase
             if ($modulo < 0) {
                 $modulo += $units[$valueUnit];
             }
-            if ($date->$valueUnit === $value ||
-                $date->$valueUnit === $modulo ||
+            if ($value === $date->$valueUnit ||
+                $modulo === $date->$valueUnit ||
                 (method_exists($date, "diffInReal$unit") && -$date->{"diffInReal$unit"}($original, false) === $value) ||
                 -$date->{"diffIn$unit"}($original, false) === $value
             ) {
@@ -694,12 +739,19 @@ class SettersTest extends AbstractTestCase
                 continue;
             }
 
-            throw new \Exception('Unhandled result for: '.
-                'Carbon::parse('.var_export($original->format('Y-m-d H:i:s.u'), true).', '.
-                var_export($original->timezoneName, true).
-                ')->addUnitNoOverflow('.implode(', ', array_map(function ($value) {
-                    return var_export($value, true);
-                }, [$valueUnit, $value, $overflowUnit])).');'."\nGetting: ".$date->format('Y-m-d H:i:s.u e'));
+            $this->failOperation(
+                $original,
+                $date,
+                $start,
+                $end,
+                'addUnitNoOverflow',
+                $valueUnit,
+                $value,
+                $overflowUnit,
+                $unit,
+                $modulo,
+                $$valueUnit
+            );
         }
 
         $minimum = static::SET_UNIT_NO_OVERFLOW_SAMPLE / 100;
@@ -728,7 +780,7 @@ class SettersTest extends AbstractTestCase
             $second = mt_rand(0, 59);
             $microsecond = mt_rand(0, 999999);
             $units = ['millennium', 'century', 'decade', 'year', 'quarter', 'month', 'day', 'hour', 'minute', 'second', 'week'];
-            $overflowUnit = $units[mt_rand(0, count($units) - 1)];
+            $overflowUnit = $units[mt_rand(0, \count($units) - 1)];
             $units = [
                 'year' => 10,
                 'month' => 12,
@@ -738,7 +790,7 @@ class SettersTest extends AbstractTestCase
                 'second' => 60,
                 'microsecond' => 1000000,
             ];
-            $valueUnit = array_keys($units)[mt_rand(0, count($units) - 1)];
+            $valueUnit = array_keys($units)[mt_rand(0, \count($units) - 1)];
             $value = mt_rand() > 0.5 ?
                 mt_rand(-9999, 9999) :
                 mt_rand(-60, 60);
@@ -757,13 +809,15 @@ class SettersTest extends AbstractTestCase
 
             $unit = ucfirst(Carbon::pluralUnit($valueUnit));
             $modulo = ($$valueUnit - $value) % $units[$valueUnit];
+
             if ($modulo < 0) {
                 $modulo += $units[$valueUnit];
             }
-            if ($date->$valueUnit === $value ||
-                $date->$valueUnit === $modulo ||
-                (method_exists($date, "diffInReal$unit") && $date->{"diffInReal$unit"}($original, false) === $value) ||
-                $date->{"diffIn$unit"}($original, false) === $value
+
+            if ($value === $date->$valueUnit ||
+                $modulo === $date->$valueUnit ||
+                (method_exists($date, "diffInReal$unit") && $value === $date->{"diffInReal$unit"}($original, false)) ||
+                $value === $date->{"diffIn$unit"}($original, false)
             ) {
                 $results['current']++;
 
@@ -782,12 +836,19 @@ class SettersTest extends AbstractTestCase
                 continue;
             }
 
-            throw new \Exception('Unhandled result for: '.
-                'Carbon::parse('.var_export($original->format('Y-m-d H:i:s.u'), true).', '.
-                var_export($original->timezoneName, true).
-                ')->subUnitNoOverflow('.implode(', ', array_map(function ($value) {
-                    return var_export($value, true);
-                }, [$valueUnit, $value, $overflowUnit])).');'."\nGetting: ".$date->format('Y-m-d H:i:s.u e'));
+            $this->failOperation(
+                $original,
+                $date,
+                $start,
+                $end,
+                'subUnitNoOverflow',
+                $valueUnit,
+                $value,
+                $overflowUnit,
+                $unit,
+                $modulo,
+                $$valueUnit
+            );
         }
 
         $minimum = static::SET_UNIT_NO_OVERFLOW_SAMPLE / 100;
@@ -797,4 +858,39 @@ class SettersTest extends AbstractTestCase
         $this->assertGreaterThan($minimum, $results['current']);
         $this->assertSame(static::SET_UNIT_NO_OVERFLOW_SAMPLE, $results['end'] + $results['start'] + $results['current']);
     }
+
+    private function failOperation(
+        Carbon $original,
+        Carbon $date,
+        Carbon $start,
+        Carbon $end,
+        string $method,
+        string $valueUnit,
+        int $value,
+        string $overflowUnit,
+        string $unit,
+        int $modulo,
+        int $variableValue
+    ): void {
+        throw new Exception(implode("\n", [
+            'Unhandled result for: '.
+            'Carbon::parse('.var_export($original->format('Y-m-d H:i:s.u'), true).', '.
+            var_export($original->timezoneName, true).
+            ")->$method(".implode(', ', array_map(function ($value) {
+                return var_export($value, true);
+            }, [$valueUnit, $value, $overflowUnit])).');',
+            'Getting: '.$date->format('Y-m-d H:i:s.u e'),
+            "Current $valueUnit: ".$date->$valueUnit,
+            'Is neither '.$start->$valueUnit." (from $start)",
+            'Nor '.$end->$valueUnit." (from $end)",
+            "Nor $value (from value)",
+            "Nor $modulo (from modulo)",
+            method_exists($date, "diffInReal$unit")
+                ? "diffInReal$unit() exists and returns ".$date->{"diffInReal$unit"}($original, false)
+                    ." while expecting $variableValue"
+                : "diffInReal$unit() does not exist",
+            "diffIn$unit() exists and returns ".$date->{"diffIn$unit"}($original, false)
+            ." while expecting $variableValue",
+        ]));
+    }
 }
