@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,8 +10,10 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\CarbonInterval;
 
+use Carbon\Carbon;
 use Carbon\CarbonInterface;
 use Carbon\CarbonInterval;
 use Symfony\Component\Translation\Loader\ArrayLoader;
@@ -46,7 +49,7 @@ class ForHumansTest extends AbstractTestCase
 
     public function testSetTranslator()
     {
-        /** @var \Carbon\Translator $t */
+        /** @var \Carbon\Translator $ori */
         $ori = CarbonInterval::getTranslator();
         $t = new Translator('fr');
         $t->addLoader('array', new ArrayLoader());
@@ -254,118 +257,253 @@ class ForHumansTest extends AbstractTestCase
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::years(2)->months(11);
-        $this->assertEquals('3 years', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('3 years', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
         $interval = CarbonInterval::years(2)->months(5)->days(70);
-        $this->assertEquals('3 years', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('3 years', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundMonths()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::months(2)->weeks(3);
-        $this->assertEquals('3 months', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('3 months', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundUp()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::days(2)->hours(23);
-        $this->assertEquals('3 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('3 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundDown()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::days(2)->hours(11);
-        $this->assertEquals('2 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('2 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundMinutes()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::days(2)->hours(11)->minutes(15);
-        $this->assertEquals('2 days 11 hours', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('2 days 11 hours', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundDownWhenNextIntervalIsNonSequential()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::days(2)->minutes(59);
-        $this->assertEquals('2 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('2 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundMultipleParts()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::days(2)->minutes(45)->seconds(59);
-        $this->assertEquals('2 days 46 minutes', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('2 days 46 minutes', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundMultiplePartsGap()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::days(2)->seconds(59);
-        $this->assertEquals('2 days 59 seconds', $interval->forHumans(['parts' => 2]));
-        $this->assertEquals('2 days 59 seconds', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
-        $this->assertEquals('2 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('2 days 59 seconds', $interval->forHumans(['parts' => 2]));
+        $this->assertSame('2 days 59 seconds', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('2 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundWeeks()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::days(13);
-        $this->assertEquals('2 weeks', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('2 weeks', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundWeeksWithMultipleParts()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::days(13);
-        $this->assertEquals('1 week 6 days', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('1 week 6 days', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundOverflowNonSequentialRoundUp()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::years(2)->months(35);
-        $this->assertEquals('5 years', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('5 years', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundOverflowNonSequentialRoundDown()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::years(2)->months(37);
-        $this->assertEquals('5 years', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('5 years', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
     }
 
     public function testRoundCarryOverDoesntMatter()
     {
         CarbonInterval::setLocale('en');
         $interval = CarbonInterval::days(2)->hours(11)->minutes(59);
-        $this->assertEquals('2 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('2 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
 
         $interval = CarbonInterval::days(2)->minutes(780);
-        $this->assertEquals('3 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('3 days', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
 
         $interval = CarbonInterval::days(2)->minutes(59)->seconds(58);
-        $this->assertEquals('2 days 1 hour', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
+        $this->assertSame('2 days 1 hour', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::ROUND]));
 
         $interval = CarbonInterval::days(2)->minutes(59)->seconds(1);
-        $this->assertEquals('2 days 1 hour', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::CEIL]));
+        $this->assertSame('2 days 1 hour', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::CEIL]));
 
         $interval = CarbonInterval::days(2)->minutes(59)->seconds(58);
-        $this->assertEquals('2 days 59 minutes', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::FLOOR]));
+        $this->assertSame('2 days 59 minutes', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::FLOOR]));
 
         // Floor is not the default behavior, see the difference below:
         $interval = CarbonInterval::days(2)->minutes(59)->seconds(62);
-        $this->assertEquals('2 days 1 hour', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::FLOOR]));
+        $this->assertSame('2 days 1 hour', $interval->forHumans(['parts' => 2, 'options' => CarbonInterface::FLOOR]));
 
         $interval = CarbonInterval::days(2)->minutes(59)->seconds(62);
-        $this->assertEquals('2 days 59 minutes', $interval->forHumans(['parts' => 2]));
+        $this->assertSame('2 days 59 minutes', $interval->forHumans(['parts' => 2]));
+
+        $start = Carbon::create(2009, 9)->startOfMonth();
+        $end = Carbon::create(2012, 2)->endOfMonth();
+
+        $interval = CarbonInterval::instance($start->diff($end))->forHumans(
+            Carbon::DIFF_ABSOLUTE,
+            false,
+            2,
+            Carbon::CEIL
+        );
+
+        $this->assertSame('2 years 6 months', $interval);
+
+        $interval = CarbonInterval::months(2)->days(30);
+        $this->assertSame('3 months', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $interval = CarbonInterval::months(2)->days(31);
+        $this->assertSame('3 months', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $interval = CarbonInterval::months(2)->days(32);
+        $this->assertSame('3 months', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::ROUND]));
+        $interval = CarbonInterval::months(2)->days(30);
+        $this->assertSame('3 months', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::CEIL]));
+        $interval = CarbonInterval::months(2)->days(31);
+        $this->assertSame('3 months', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::CEIL]));
+        $interval = CarbonInterval::months(2)->days(32);
+        $this->assertSame('4 months', $interval->forHumans(['parts' => 1, 'options' => CarbonInterface::CEIL]));
+    }
+
+    public function testCeilShortMonth()
+    {
+        Carbon::setTestNow('2022-02-08T10:27:03Z');
+        $this->assertSame(
+            '4 weeks ago',
+            Carbon::parse('2022-01-11 15:36:29')->diffForHumans(['parts' => 1, 'options' => Carbon::CEIL])
+        );
+        $this->assertSame(
+            '1 month ago',
+            Carbon::parse('2022-01-10 16:57:38')->diffForHumans(['parts' => 1, 'options' => Carbon::CEIL])
+        );
+    }
+
+    public function testSkipUnits()
+    {
+        CarbonInterval::setLocale('en');
+        $interval = CarbonInterval::days(15)->hours(11)->minutes(15);
+        $this->assertSame('15 days 11 hours', $interval->forHumans([
+            'parts' => 2,
+            'skip' => 'week',
+        ]));
+
+        $this->assertSame('15 days 675 minutes', $interval->forHumans([
+            'parts' => 2,
+            'skip' => ['weeks', 'hours'],
+        ]));
+
+        $interval = CarbonInterval::days(15)->hours(11)->minutes(15);
+        $this->assertSame('15 days 675 minutes', $interval->forHumans([
+            'parts' => 2,
+            'skip' => ['weeks', 'hours'],
+        ]));
+
+        $factors = CarbonInterval::getCascadeFactors();
+
+        CarbonInterval::setCascadeFactors(['weeks' => [5, 'days']]);
+
+        $interval = CarbonInterval::days(15)->hours(11)->minutes(15);
+        $this->assertSame('15 days 675 minutes', $interval->forHumans([
+            'parts' => 2,
+            'skip' => ['weeks', 'hours'],
+        ]));
+
+        CarbonInterval::setCascadeFactors([
+            'weeks' => [5, 'days'],
+            'hours' => [30, 'minutes'],
+        ]);
+
+        $interval = CarbonInterval::days(15)->hours(11)->minutes(15);
+        $this->assertSame('15 days 345 minutes', $interval->forHumans([
+            'parts' => 2,
+            'skip' => ['weeks', 'hours'],
+        ]));
+
+        CarbonInterval::setCascadeFactors($factors);
     }
 
     public function testGetValuesSequence()
     {
         $this->assertSame([], CarbonInterval::days(0)->getValuesSequence());
     }
+
+    public function testMinimumUnitDefault()
+    {
+        CarbonInterval::setLocale('en');
+        $interval = CarbonInterval::fromString('1 second 114 milliseconds');
+        $this->assertSame('1 second', $interval->forHumans(['parts' => 3]));
+    }
+
+    public function testMinimumUnitHours()
+    {
+        CarbonInterval::setLocale('en');
+        $interval = CarbonInterval::fromString('1 hour 1 second 114 milliseconds');
+        $this->assertSame('1 hour', $interval->forHumans(['parts' => 3, 'minimumUnit' => 'hour']));
+    }
+
+    public function testMinimumUnitMillisecondsShort()
+    {
+        CarbonInterval::setLocale('en');
+        $interval = CarbonInterval::fromString('1 second 114 milliseconds');
+        $this->assertSame('1s 114ms', $interval->forHumans(['parts' => 3, 'short' => true, 'minimumUnit' => 'ms']));
+    }
+
+    public function testMinimumUnitMicroseconds()
+    {
+        CarbonInterval::setLocale('fr');
+        $interval = CarbonInterval::fromString('1s 114ms 584µs');
+        $this->assertEquals(
+            '1 seconde, 114 millisecondes et 584 microsecondes',
+            $interval->forHumans(['parts' => 3, 'join' => true, 'minimumUnit' => 'µs'])
+        );
+    }
+
+    public function testMinimumUnitMillisecondsInFrenchAndGerman()
+    {
+        $interval = CarbonInterval::fromString('98756 milliseconds')->cascade();
+        $this->assertEquals(
+            'une minute, 38 secondes et 756 millisecondes',
+            $interval->locale('fr')->forHumans(['parts' => 3, 'minimumUnit' => 'ms', 'join' => true, 'aUnit' => true])
+        );
+        $this->assertEquals(
+            'eine Minute, 38 Sekunden und 756 Millisekunden',
+            $interval->locale('de')->forHumans(['parts' => 3, 'minimumUnit' => 'ms', 'join' => true, 'aUnit' => true])
+        );
+    }
+
+    public function testMinimumUnitNoInterval()
+    {
+        CarbonInterval::setLocale('en');
+        $interval = CarbonInterval::fromString('1 second 114 milliseconds');
+        // Test with and without NO_ZERO_DIFF
+        $this->assertSame('1 hour', $interval->forHumans(['parts' => 3, 'minimumUnit' => 'hour', 'options' => CarbonInterface::NO_ZERO_DIFF]));
+        $this->assertSame('0 hours', $interval->forHumans(['parts' => 3, 'minimumUnit' => 'hour', 'options' => 0]));
+    }
 }
