@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,6 +10,7 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\CarbonPeriod;
 
 use Carbon\CarbonPeriod;
@@ -21,7 +23,7 @@ class CloneTest extends AbstractTestCase
         $period = CarbonPeriod::create('R4/2012-07-01T00:00:00/P7D');
         $clone = $period->clone();
 
-        $this->assertSame(strval($period), strval($clone));
+        $this->assertSame((string) $period, (string) $clone);
         $this->assertNotSame($period, $clone);
         $this->assertEquals($period, $clone);
     }
@@ -31,7 +33,7 @@ class CloneTest extends AbstractTestCase
         $period = CarbonPeriod::create('R4/2012-07-01T00:00:00/P7D');
         $clone = $period->copy();
 
-        $this->assertSame(strval($period), strval($clone));
+        $this->assertSame((string) $period, (string) $clone);
         $this->assertNotSame($period, $clone);
         $this->assertEquals($period, $clone);
     }
