@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,32 +10,21 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\CarbonPeriod;
 
+use BadMethodCallException;
 use Carbon\Carbon;
 use Carbon\CarbonPeriod;
 use Tests\AbstractTestCase;
 
 class StrictModeTest extends AbstractTestCase
 {
-    protected function setUp(): void
-    {
-        parent::setUp();
-        Carbon::useStrictMode();
-    }
-
-    protected function tearDown(): void
-    {
-        parent::tearDown();
-        Carbon::useStrictMode();
-    }
-
     public function testCallWithStrictMode()
     {
-        $this->expectException(\BadMethodCallException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new BadMethodCallException(
             'Method foobar does not exist.'
-        );
+        ));
 
         /** @var mixed $period */
         $period = CarbonPeriod::create();
