@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,9 +10,11 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\CarbonTimeZone;
 
 use Carbon\CarbonTimeZone;
+use Carbon\Exceptions\InvalidTimeZoneException;
 use Tests\AbstractTestCase;
 use Tests\CarbonTimeZone\Fixtures\UnknownZone;
 
@@ -26,6 +29,22 @@ class CreateTest extends AbstractTestCase
         $tz = CarbonTimeZone::create(6);
 
         $this->assertSame('+06:00', $tz->getName());
+
+        $tz = CarbonTimeZone::create('+01');
+
+        $this->assertSame('+01:00', $tz->getName());
+
+        $tz = new CarbonTimeZone('+01');
+
+        $this->assertSame('+01:00', $tz->getName());
+
+        $tz = CarbonTimeZone::create('-01');
+
+        $this->assertSame('-01:00', $tz->getName());
+
+        $tz = new CarbonTimeZone('-01');
+
+        $this->assertSame('-01:00', $tz->getName());
     }
 
     public function testInstance()
@@ -44,10 +63,9 @@ class CreateTest extends AbstractTestCase
 
     public function testSafeCreateDateTimeZoneWithoutStrictMode()
     {
-        $this->expectException(\InvalidArgumentException::class);
-        $this->expectExceptionMessage(
+        $this->expectExceptionObject(new InvalidTimeZoneException(
             'Absolute timezone offset cannot be greater than 100.'
-        );
+        ));
 
         new CarbonTimeZone(-15e15);
     }
