@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 
 /**
@@ -9,12 +10,15 @@ declare(strict_types=1);
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+
 namespace Tests\CarbonTimeZone\Fixtures;
 
 use Carbon\CarbonTimeZone;
+use ReturnTypeWillChange;
 
 class UnknownZone extends CarbonTimeZone
 {
+    #[ReturnTypeWillChange]
     public function getName()
     {
         return 'foobar';
